package com.muyu.data.source.domain;

import java.util.Date;
import com.fasterxml.jackson.annotation.JsonFormat;
import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import com.muyu.common.security.utils.SecurityUtils;
import com.muyu.common.system.domain.LoginUser;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.NoArgsConstructor;
import lombok.AllArgsConstructor;
import lombok.experimental.SuperBuilder;
import io.swagger.annotations.*;
import com.muyu.common.core.annotation.Excel;
import com.muyu.data.source.domain.req.DataSourceQueryReq;
import com.muyu.data.source.domain.req.DataSourceSaveReq;
import com.muyu.data.source.domain.req.DataSourceEditReq;
import com.muyu.common.core.web.domain.BaseEntity;

/**
 * 数据源对象 data_source
 *
 * @author muyu
 * @date 2024-04-21
 */
@Data
@SuperBuilder
@NoArgsConstructor
@AllArgsConstructor
@TableName("data_source")
@EqualsAndHashCode(callSuper = true)
@ApiModel(value = "DataSource", description = "数据源")
public class DataSource extends BaseEntity {

    private static final long serialVersionUID = 1L;

    /** 数据管理id */
    @TableId(value = "id",type = IdType.AUTO)
    @ApiModelProperty(name = "数据管理id", value = "数据管理id")
    private Long id;

    /** 接入源名称 */
    @Excel(name = "接入源名称")
    @ApiModelProperty(name = "接入源名称", value = "接入源名称")
    private String accessSourceName;

    /** 数据来源系统名称 */
    @Excel(name = "数据来源系统名称")
    @ApiModelProperty(name = "数据来源系统名称", value = "数据来源系统名称")
    private String dataSourceSystemName;

    /** 主机地址 */
    @Excel(name = "主机地址")
    @ApiModelProperty(name = "主机地址", value = "主机地址")
    private String hostAddress;

    /** 主机端口 */
    @Excel(name = "主机端口")
    @ApiModelProperty(name = "主机端口", value = "主机端口")
    private String hostPort;

    /** 数据连接类型ID */
    @Excel(name = "数据连接类型ID")
    @ApiModelProperty(name = "数据连接类型ID", value = "数据连接类型ID")
    private Long dataAccessTypeId;

    /** 数据库名称 */
    @Excel(name = "数据库名称")
    @ApiModelProperty(name = "数据库名称", value = "数据库名称")
    private String databaseName;

    /** 数据连接参数 */
    @Excel(name = "数据连接参数")
    @ApiModelProperty(name = "数据连接参数", value = "数据连接参数")
    private String dataConnectionParameter;

    /** 初始连接数量 */
    @Excel(name = "初始连接数量")
    @ApiModelProperty(name = "初始连接数量", value = "初始连接数量")
    private String initialQuantity;

    /** 最大连接数量 */
    @Excel(name = "最大连接数量")
    @ApiModelProperty(name = "最大连接数量", value = "最大连接数量")
    private String maximumQuantity;

    /** 最大等待时间 */
    @Excel(name = "最大等待时间")
    @ApiModelProperty(name = "最大等待时间", value = "最大等待时间")
    private Long maximumTime;

    /** 最大等待次数 */
    @Excel(name = "最大等待次数")
    @ApiModelProperty(name = "最大等待次数", value = "最大等待次数")
    private String maximumFrequency;

    /** 数据库用户名 */
    @Excel(name = "数据库用户名")
    @ApiModelProperty(name = "数据库用户名", value = "数据库用户名")
    private String databaseUserName;

    /** 数据库用户密码 */
    @Excel(name = "数据库用户密码")
    @ApiModelProperty(name = "数据库用户密码", value = "数据库用户密码")
    private String databaseUserPassword;

    /**
    * 查询构造器
    */
    public static DataSource queryBuild( DataSourceQueryReq dataSourceQueryReq){
        return DataSource.builder()
                .accessSourceName(dataSourceQueryReq.getAccessSourceName())
                .dataSourceSystemName(dataSourceQueryReq.getDataSourceSystemName())
                .hostAddress(dataSourceQueryReq.getHostAddress())
                .hostPort(dataSourceQueryReq.getHostPort())
                .dataAccessTypeId(dataSourceQueryReq.getDataAccessTypeId())
                .databaseName(dataSourceQueryReq.getDatabaseName())
                .dataConnectionParameter(dataSourceQueryReq.getDataConnectionParameter())
                .initialQuantity(dataSourceQueryReq.getInitialQuantity())
                .maximumQuantity(dataSourceQueryReq.getMaximumQuantity())
                .maximumTime(dataSourceQueryReq.getMaximumTime())
                .maximumFrequency(dataSourceQueryReq.getMaximumFrequency())
                .build();
    }

    /**
    * 添加构造器
    */
    public static DataSource saveBuild(DataSourceSaveReq dataSourceSaveReq){
        return DataSource.builder()
                .accessSourceName(dataSourceSaveReq.getAccessSourceName())
                .dataSourceSystemName(dataSourceSaveReq.getDataSourceSystemName())
                .hostAddress(dataSourceSaveReq.getHostAddress())
                .hostPort(dataSourceSaveReq.getHostPort())
                .dataAccessTypeId(dataSourceSaveReq.getDataAccessTypeId())
                .databaseName(dataSourceSaveReq.getDatabaseName())
                .dataConnectionParameter(dataSourceSaveReq.getDataConnectionParameter())
                .initialQuantity(dataSourceSaveReq.getInitialQuantity())
                .maximumQuantity(dataSourceSaveReq.getMaximumQuantity())
                .maximumTime(dataSourceSaveReq.getMaximumTime())
                .maximumFrequency(dataSourceSaveReq.getMaximumFrequency())
                .databaseUserName(dataSourceSaveReq.getDatabaseUserName())
                .databaseUserPassword(dataSourceSaveReq.getDatabaseUserPassword())
                .remark(dataSourceSaveReq.getRemark())
                .createBy(SecurityUtils.getUsername())
                .createTime(new Date())
                .build();
    }

    /**
    * 修改构造器
    */
    public static DataSource editBuild(Long id, DataSourceEditReq dataSourceEditReq){
        return DataSource.builder()
            .id(id)
                .accessSourceName(dataSourceEditReq.getAccessSourceName())
                .dataSourceSystemName(dataSourceEditReq.getDataSourceSystemName())
                .hostAddress(dataSourceEditReq.getHostAddress())
                .hostPort(dataSourceEditReq.getHostPort())
                .dataAccessTypeId(dataSourceEditReq.getDataAccessTypeId())
                .databaseName(dataSourceEditReq.getDatabaseName())
                .dataConnectionParameter(dataSourceEditReq.getDataConnectionParameter())
                .initialQuantity(dataSourceEditReq.getInitialQuantity())
                .maximumQuantity(dataSourceEditReq.getMaximumQuantity())
                .maximumTime(dataSourceEditReq.getMaximumTime())
                .maximumFrequency(dataSourceEditReq.getMaximumFrequency())
                .databaseUserName(dataSourceEditReq.getDatabaseUserName())
                .databaseUserPassword(dataSourceEditReq.getDatabaseUserPassword())
                .updateBy(SecurityUtils.getUsername())
                .updateTime(new Date())
                .build();
    }

}
